@@ -1,9 +1,8 @@
 import dataclasses
-from dataclasses import dataclass
-from enum import Enum
-
 import json
+from dataclasses import dataclass
 from datetime import datetime
+from enum import Enum
 from pathlib import Path
 from typing import List, Union, Dict
 
@@ -745,7 +744,7 @@ class FileDefinition:
 
 
     Attributes:
-        name: Table / file name.
+        name: File name.
         full_path (str): (optional) Full path of the file.
         tags (list):
                 List of tags that are assigned to this file
@@ -756,6 +755,17 @@ class FileDefinition:
         notify: Notifies project administrators that a file was uploaded.
 
     """
+    SYSTEM_TAG_PREFIXES = ['componentId:',
+                           'configurationId:',
+                           'configurationRowId:',
+                           'runId:',
+                           'branchId:']
+
+    OUTPUT_MANIFEST_KEYS = ["tags",
+                            "is_public",
+                            "is_permanent",
+                            "is_encrypted",
+                            "notify"]
 
     def __init__(self, full_path: str,
                  tags: List[str] = None,
@@ -766,7 +776,7 @@ class FileDefinition:
         """
 
         Args:
-            full_path (str): (optional) Full path of the file.
+            full_path (str): Full path of the file.
             tags (list):
                 List of tags that are assigned to this file
             is_public: When true, the file URL will be permanent and publicly accessible.
@@ -775,16 +785,8 @@ class FileDefinition:
             is_encrypted: If true, the file content will be encrypted in the storage.
             notify: Notifies project administrators that a file was uploaded.
         """
-        self._raw_manifest = dict()
+        self._raw_manifest: dict = dict()
         self.full_path = full_path
-        # separate id from name
-        name = Path(full_path).name
-        fsplit = name.split('_', 2)
-        if len(fsplit) > 1:
-            self._raw_manifest['id'] = fsplit[0]
-            self.name = fsplit[1]
-        else:
-            self.name = name
 
         self.tags = tags
         self.is_public = is_public
@@ -792,6 +794,9 @@ class FileDefinition:
         self.is_encrypted = is_encrypted
         self.notify = notify
 
+        # infer stage by default
+        self.__stage = self.__get_stage_inferred()
+
     @classmethod
     def build_from_manifest(cls,
                             manifest_file_path: str
@@ -830,28 +835,110 @@ class FileDefinition:
 
         return file_def
 
-    def get_manifest_dictionary(self) -> dict:
+    def get_manifest_dictionary(self, manifest_type='out') -> dict:
         """
 
-        Args:
-            manifest_type (str): either 'in' or 'out'. This option keeps only values that are applicable for
+        Returns manifest dictionary in appropriate manifest_type: either 'in' or 'out'.
+        By default returns output manifest.
+             The result keeps only values that are applicable for
              the selected type of the Manifest file. Because although input and output manifests share most of
              the attributes, some are not shared.
 
              See [manifest files](https://developers.keboola.com/extend/common-interface/manifest-files)
              for more information.
 
+        Args:
+            manifest_type (str): either 'in' or 'out'.
+
         Returns:
             dict representation of the manifest file in a format expected / produced by the Keboola Connection
 
         """
 
-        return self._raw_manifest
+        return self.__get_raw_manifest_by_type(manifest_type)
+
+    def __get_raw_manifest_by_type(self, manifest_type):
+        new_manifest = {}
+        if manifest_type == 'out':
+            new_manifest = self.__build_out_manifest()
+        else:
+            new_manifest = self._raw_manifest
+
+        return new_manifest
+
+    def __build_out_manifest(self):
+        new_manifest = {}
+        for key in self.OUTPUT_MANIFEST_KEYS:
+            if key in self._raw_manifest:
+                new_manifest[key] = self._raw_manifest[key]
+
+        return new_manifest
+
+    def __get_stage_inferred(self):
+        stage = 'out'
+        if Path(self.full_path).parent.parent.name == 'in':
+            stage = 'in'
+        elif Path(self.full_path).parent.parent.name == 'out':
+            stage = 'out'
+        return stage
+
+    @classmethod
+    def is_system_tag(cls, tag: str) -> bool:
+        for prefix in cls.SYSTEM_TAG_PREFIXES:
+            if tag.startswith(prefix):
+                return True
+        return False
+
+    @property
+    def name(self):
+        """
+        File name - excluding the KBC ID if present (`str`, read-only)
+        """
+        # separate id from name
+        file_name = Path(self.full_path).name
+        if self._raw_manifest.get('id'):
+            fsplit = file_name.split('_', 1)
+            if len(fsplit) > 1:
+                self._raw_manifest['id'] = fsplit[0]
+                file_name = fsplit[1]
+        return file_name
+
+    @property
+    def full_name(self):
+        """
+        File name - full file name, directly from the path. Includes the KBC generated ID. (`str`, read-only)
+        """
+        return Path(self.full_path).name
+
+    @property
+    def stage(self) -> str:
+        """
+        Helper property marking the stage of the file. (str)
+        """
+        return self.__stage
+
+    @stage.setter
+    def stage(self, stage: str):
+        if stage not in ['in', 'out']:
+            raise ValueError(f'Invalid stage "{stage}", supported values are: "in", "out"')
+        self.__stage = stage
 
     # ########### Output manifest properties - R/W
 
+    @property
+    def user_tags(self) -> List[str]:
+        """
+        User defined tags excluding the system tags
+        """
+        # filter system tags
+        tags: List[str] = [tag for tag in self._raw_manifest.get('tags', []) if not self.is_system_tag(tag)]
+        return tags
+
     @property
     def tags(self) -> List[str]:
+        """
+        All tags specified on the file
+        """
         return self._raw_manifest.get('tags', [])
 
     @tags.setter
@@ -892,14 +979,6 @@ class FileDefinition:
     def notify(self, notify: bool):
         self._raw_manifest['notify'] = notify
 
-    @property
-    def name(self) -> str:
-        return self.__name
-
-    @name.setter
-    def name(self, val: str):
-        self.__name = val
-
     # ########### Input manifest properties - Read ONLY
     @property
     def id(self) -> str:  # File ID in the KBC Storage (read only input attribute)
